using System;
using System.Windows.Forms;
using System.Drawing;
using EasyImages;

// Author: Saul Greenberg and Mark Watson, University of Calgary, http://www.cpsc.ucalgary.ca/~saul  
// Copyright: (c) June 7, 2007.
// Documentation: see http://grouplab.cpsc.ucalgary.ca/cookbook/ and select EasyImages. 
// The API Documentation and a tutorial explaining this and other programs are available there
// License: See License details included in the distribution package. Essentialy, permission to 
// use and/or alter this program for non-commerical and/or educational purposes is granted,
// as long as attribution to the above author is maintained.

namespace CameraMinimal
{
  public partial class Form1 : Form
  {
    private EasyImages.CameraClient camera;
    private delegate void SetPictureBoxImage(PictureBox pbox, Bitmap image);
   
    public Form1()
    {
      InitializeComponent();
    }

    private void Form1_Load(object sender, EventArgs e)
    {
      camera = new EasyImages.CameraClient("DefaultCamera");
      camera.ReceivedFrame += new CameraClient.ReceivedFrameEventHandler(camera_ReceivedFrame);
      camera.Start();
    }

    private void camera_ReceivedFrame(object sender, CameraEventArgs e)
    {
      this.DisplayImageInPictureBox(pictureBox1, e.Bitmap);
    }

    // Displaying an image in a picture box may fail occassionally due to threading issues.
    // To resolve this, we check to see what thread we are in, and invoke a picture box 
    // delegate if we are in the wrong thread. Note that this is the recommended way to 
    // do things like this in .NET, and is not something imposed by EasyImages.
    private void DisplayImageInPictureBox(PictureBox pbox, Image image)
    {
      if (pbox.InvokeRequired) // We are in the wrong thread. Call ourselves in the correct thread
      {
        SetPictureBoxImage theDelegate = new SetPictureBoxImage(DisplayImageInPictureBox);
        BeginInvoke(theDelegate, new object[] { pbox, image });  
      }
      else // we are in the correct thread, so assign the image
      {
        pbox.Image = image; 
      }
    }
  }
}
