using System;
using System.Drawing;
using System.Windows.Forms;
using EasyImages;
// Author: Mark Watson, modified by Saul Greenberg, University of Calgary, http://www.cpsc.ucalgary.ca/~saul  
// Copyright: (c) June 7, 2007.
// Documentation: see http://grouplab.cpsc.ucalgary.ca/cookbook/ and select EasyImages. 
// The API Documentation and a tutorial explaining this and other programs are available there
// License: See License details included in the distribution package. Essentialy, permission to 
// use and/or alter this program for non-commerical and/or educational purposes is granted,
// as long as attribution to the above author is maintained.
namespace VideoAlphaBlending
{
  public partial class Form1 : Form
  {
    EasyImages.CameraClient camera;
    Bitmap BlendedImage; // keep a copy of the last frame
    private delegate void SetPictureBoxImage(PictureBox pbox, Bitmap image);

    public Form1()
    {
      InitializeComponent();
    }

    //As the GUI is loaded, create a new camera object and set its initial properties
    private void Form1_Load(object sender, EventArgs e)
    {
      camera = new EasyImages.CameraClient("DefaultCamera");
      camera.FramesPerSecond = 10;
      camera.ReceivedFrame += new EasyImages.CameraClient.ReceivedFrameEventHandler (camera_ReceivedFrame);
      camera.Start();
    }

    // When we receive a frame, alpha blend it with the last f
    void camera_ReceivedFrame(object sender, EasyImages.CameraEventArgs e)
    {
      Bitmap Temp;
      // If we don't have a blended image (i.e., if its the first frame), 
      // then just make it the video frame
      if (BlendedImage == null) {
        Temp = e.Bitmap.Clone() as Bitmap;
      } else {
        // First, create a copy of the current frame
         Temp = e.Bitmap.Clone() as Bitmap; 
        // Then blend it with the previous image
        EasyImages.ImageManipulator.AlphaBlend(ref Temp, BlendedImage, (float).15);
      }
      BlendedImage = Temp;
      DisplayImageInPictureBox (this.pbImage, BlendedImage);
    }

    //Display the image in the picture box in the correct thread
    private void DisplayImageInPictureBox(PictureBox pbox, Image image)
    {
      if (pbox.InvokeRequired) // We are in the wrong thread. Call ourselves in the correct thread
      {
        SetPictureBoxImage theDelegate = new SetPictureBoxImage(DisplayImageInPictureBox);
        BeginInvoke(theDelegate, new object[] { pbox, image });
      }
      else // we are in the correct thread, so assign the image
      {
        pbox.Image = image;
      }
    }
  }
}