using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using EasyImages;

// Author: Saul Greenberg and Edward Tse
// University of Calgary, http://www.cpsc.ucalgary.ca/~saul  
// Copyright: (c) June 7, 2007.
// Documentation: see http://grouplab.cpsc.ucalgary.ca/cookbook/ and select EasyImages. 
// The API Documentation and a tutorial explaining this and other programs are available there
// License: See License details included in the distribution package. Essentialy, permission to 
// use and/or alter this program for non-commerical and/or educational purposes is granted,
// as long as attribution to the above author is maintained.
namespace VideoResize
{
  public partial class Form1 : Form
  {
    private EasyImages.CameraClient camera;
    private delegate void SetPictureBoxImage(PictureBox pbox, Bitmap image);

    public Form1()
    {
      InitializeComponent();
    }
    private void Form1_Load(object sender, EventArgs e)
    {
      camera = new EasyImages.CameraClient("DefaultCamera");
      camera.FramesPerSecond = 10;
      camera.ReceivedFrame += new EasyImages.CameraClient.ReceivedFrameEventHandler(wc_ReceivedFrame);
      camera.Start();
    }
    void wc_ReceivedFrame(object sender, EasyImages.CameraEventArgs e)
    {
      drawImage(e.Bitmap);
    }


    //Display the bitmap, and the resizing label at its bottom right corner
    private void drawImage(Bitmap bm)
    {
      DisplayImageInPictureBox (pbImage, EasyImages.ImageGenerator.Resize(pbImage.Width, pbImage.Height, bm));
      lblDrag.Top = pbImage.Bottom;
      lblDrag.Left = pbImage.Right;
    }

    private void DisplayImageInPictureBox(PictureBox pbox, Image image)
    {
      if (pbox.InvokeRequired) // We are in the wrong thread. Call ourselves in the correct thread
      {
        SetPictureBoxImage theDelegate = new SetPictureBoxImage(DisplayImageInPictureBox);
        BeginInvoke(theDelegate, new object[] { pbox, image });
      }
      else // we are in the correct thread, so assign the image
      {
        pbox.Image = image;
      }
    }

    // The code below just resets the size of the pbImage picture box.
    // As a side effect, the PictureBox resize event is triggered, and the
    // frame is redrawn within it at the correct size.
    private bool isDragging = false;
    private Point draggingOffset = new Point(0, 0);
    private void pbImage_MouseDown(object sender, MouseEventArgs e)
    {
      if ((e.X > pbImage.Width - 30) && (e.Y > pbImage.Height - 30))
      {
        draggingOffset = new Point(pbImage.Width - e.X, pbImage.Height - e.Y);
        isDragging = true;
      }
    }

    //Resize, but esure this image is at least 10 pixels in size in any dimension
    private void pbImage_MouseMove(object sender, MouseEventArgs e)
    {
      if (isDragging)
      {
        int w = (e.X + draggingOffset.X < 10) ? 10 : e.X + draggingOffset.X;
        int h = (e.Y + draggingOffset.Y < 10) ? 10 : e.Y + draggingOffset.Y;
        pbImage.Size = new Size(w, h);
      }
    }

    private void pbImage_MouseUp(object sender, MouseEventArgs e)
    {
      if (isDragging)
      {
        isDragging = false;
      }
    }

    // If we wanted to do it interactively, we could draw the image as
    // the picturebox is being resized. But this is redundant; since we are getting new video frames
    // already at 15fps,  this is sufficient to animate the resizing
    // so it looks fluid.
    private void pbImage_SizeChanged(object sender, EventArgs e)
    {
      // drawImage((Bitmap) pbImage.Image);
    }
  }
}
