using System;
using System.Drawing;
using System.Windows.Forms;
using EasyImages;

// Author: Saul Greenberg, University of Calgary, http://www.cpsc.ucalgary.ca/~saul  
// Copyright: (c) June 7, 2007.
// Documentation: see http://grouplab.cpsc.ucalgary.ca/cookbook/ and select EasyImages. 
// The API Documentation and a tutorial explaining this and other programs are available there
// License: See License details included in the distribution package. Essentialy, permission to 
// use and/or alter this program for non-commerical and/or educational purposes is granted,
// as long as attribution to the above author is maintained.
namespace ImageManipulator
{
  public partial class Form1 : Form
  {
    EasyImages.CameraClient camera;
    private delegate void SetPictureBoxImage(PictureBox pbox, Bitmap image);

    public Form1()
    {
        InitializeComponent();
    }
    //As the GUI is loaded, create a new camera object and set its initial properties
    private void Form1_Load(object sender, EventArgs e)
    {
         //Create a new camera object and set its initial properties
        camera = new EasyImages.CameraClient("DefaultCamera");
        camera.FramesPerSecond = 15;
        camera.ReceivedFrame += new CameraClient.ReceivedFrameEventHandler(camera_ReceivedFrame);
        camera.Start();
        CreateListbox();
    }

    // Manipulate the frame according to the user' selections
    void camera_ReceivedFrame(object sender, CameraEventArgs e)
    {
        manipulateFrame(e.Bitmap);
        DisplayImageInPictureBox(pbCamera, e.Bitmap);
    }
 
    // Populate the listboxes with  the dither and distortion styles
    // We exploit the Enumerator to do this.
    void CreateListbox()
    {
      // Add the various Dither styles to the Dither listbox
      lbDither.Items.Add("None");
      foreach (string style in Enum.GetNames(typeof(EasyImages.DitherStyle)))
      {
        lbDither.Items.Add(style);
      }
      lbDither.SetSelected(0, true); 
          
      //Add the varioous Distortion styles to the Distortion listbox
      lbDistortions.Items.Add("None"); 
      foreach (string style in Enum.GetNames(typeof(EasyImages.DistortionStyle)))
      {
        lbDistortions.Items.Add(style);
      } 
      lbDistortions.SetSelected(0, true);
    }

    // Given a bitmap, alter it to match the user's selection of Dither and/or Distortion style 
    // We used Enums to make this trivial,
    void manipulateFrame(Bitmap im)
    {
      // Don't do anything if the Don't Distort checkbox is selected, or 
      if (cbDontDistort.Checked)  return;

      //Find out what dither style was selected and apply it to the image
      string choice = lbDither.SelectedItem.ToString();
      if (choice != "None" )
       EasyImages.ImageManipulator.Dither ((EasyImages.DitherStyle)Enum.Parse(typeof(EasyImages.DitherStyle), choice), im);

      //Find out what distortion style was selected and apply it to the image
      choice = lbDistortions.SelectedItem.ToString();
      if (choice != "None") 
        EasyImages.ImageManipulator.Distort((EasyImages.DistortionStyle)Enum.Parse(typeof(EasyImages.DistortionStyle), choice), im); 
    }

    //Display the image in the picture box in the correct thread
    private void DisplayImageInPictureBox(PictureBox pbox, Image image)
    {
      if (pbox.InvokeRequired) // We are in the wrong thread. Call ourselves in the correct thread
      {
        SetPictureBoxImage theDelegate = new SetPictureBoxImage(DisplayImageInPictureBox);
        BeginInvoke(theDelegate, new object[] { pbox, image });
      }
      else // we are in the correct thread, so assign the image
      {
        pbox.Image = image;
      }
    }
  }
}